import cv2
from typing import Iterable

from darcyai_engine.input.input_stream import InputStream
from darcyai_engine.stream_data import StreamData
from darcyai_engine.utils import validate_not_none, timestamp, validate_type


class VideoFileStream(InputStream):
    """
    Args:
        file_name: The name of the video file to stream.
    """

    def __init__(self, file_name: str):
        super().__init__()

        validate_not_none(file_name, "file_name is required")
        validate_type(file_name, str, "file_name must be a string")

        self.__file_name = file_name

        self.__vs = None
        self.__stopped = True

    def stop(self) -> None:
        """
        Stops the video stream.

        Args:
            None

        Returns:
            None
        """
        if self.__vs is not None:
            self.__vs.release()
            self.__vs = None

        self.__stopped = True

    def stream(self) -> Iterable[StreamData]:
        """
        Streams the video frames.

        Args:
            None

        Returns:
            An iterable of StreamData objects.
        """
        if self.__stopped or self.__vs is None:
            self.__vs = self.__initialize_video_file_stream()

        self.__stopped = False

        while not self.__stopped and self.__vs.isOpened():
            success, frame = self.__vs.read()
            if not success:
                break

            yield(StreamData(frame, timestamp()))

    def __initialize_video_file_stream(self) -> cv2.VideoCapture:
        cap = cv2.VideoCapture(self.__file_name)

        if not cap.isOpened():
            raise Exception(f"Cannot open {self.__file_name}")

        return cap
