import csv
import io

from darcyai_engine.file_stream import FileStream
from darcyai_engine.output.output_stream import OutputStream
from darcyai_engine.utils import validate_not_none, validate_type


class CSVOutputStream(OutputStream):
    """
    OutputStream implementation that writes to a CSV file.

    Args:
        file_path (str): The path to the CSV file to write to.

    Kwargs:
        delimiter (str): The delimiter to use in the CSV file.
        quotechar (str): The quote character to use in the CSV file.
        buffer_size (int): The size of the buffer to use when writing to the
            CSV file.
        flush_interval (int): The number of seconds before flushing
            the buffer to disk.
    """

    def __init__(
            self,
            file_path: str,
            delimiter: str = ",",
            quotechar: str = "|",
            buffer_size: int = 0,
            flush_interval: int = 0) -> None:
        super().__init__()

        validate_not_none(file_path, "file_path is required")
        validate_type(file_path, str, "file_path must be a string")

        validate_not_none(delimiter, "delimiter is required")
        validate_type(delimiter, str, "delimiter must be a string")

        validate_not_none(quotechar, "quotechar is required")
        validate_type(quotechar, str, "quotechar must be a string")

        self.__file_stream = FileStream(
            file_path,
            buffer_size=buffer_size,
            flush_interval=flush_interval)

        self.__delimiter = delimiter
        self.__quotechar = quotechar
        self.__delimiter = delimiter
        self.__quotechar = quotechar

        self.config_schema = []

    def write(self, data: list) -> None:
        """
        Writes the given data to the CSV file.

        Args:
            data (list): The data to write to the CSV file.

        Returns:
            None
        """
        if data is None:
            return

        validate_type(data, list, "data must be a list")

        output = io.StringIO()
        csv_writer = csv.writer(
            output,
            delimiter=self.__delimiter,
            quotechar=self.__quotechar)

        csv_writer.writerow(data)

        self.__file_stream.write_string(output.getvalue())

    def close(self) -> None:
        """
        Closes the CSV file.

        Args:
            None

        Returns:
            None
        """
        self.__file_stream.close()
