import json

from darcyai_engine.file_stream import FileStream
from darcyai_engine.output.output_stream import OutputStream
from darcyai_engine.utils import validate_not_none, validate_type


class JSONOutputStream(OutputStream):
    """
    OutputStream implementation that writes to a JSON file.

    Args:
        file_path (str): The path to the JSON file to write to.

    Kwargs:
        buffer_size (int): The size of the buffer to use when writing to the
            JSON file.
        flush_interval (int): The number of seconds before flushing
            the buffer to disk.
    """

    def __init__(self,
                 file_path: str,
                 buffer_size: int = 0,
                 flush_interval: int = 0) -> None:
        super().__init__()

        validate_not_none(file_path, "file_path is required")
        validate_type(file_path, str, "file_path must be a string")

        self.__file_stream = FileStream(
            file_path,
            buffer_size=buffer_size,
            flush_interval=flush_interval)

        self.config_schema = []

    def write(self, data: dict) -> None:
        """
        Writes the given data to the JSON file.

        Args:
            data (dict): The data to write to the JSON file.

        Returns:
            None
        """
        if data is None:
            return

        validate_type(data, dict, "data must be a dictionary")

        self.__file_stream.write_string(json.dumps(data))

    def close(self) -> None:
        """
        Closes the JSON file.

        Args:
            None

        Returns:
            None
        """
        self.__file_stream.close()
