from typing import Any

from darcyai_engine.configurable import Configurable
from darcyai_engine.config_registry import ConfigRegistry
from darcyai_engine.utils import validate_not_none, validate_type


class OutputStream(Configurable):
    """
    OutputStream is a class that is used to write output to a stream.

    Kwargs:
        ignore_none (bool): Whether or not to call the endpoint when
            the data is None.
    """

    def __init__(self, ignore_none: bool = True, **kwargs):
        """
        Initializes the OutputStream.
        """
        Configurable.__init__(self, **kwargs)


        validate_not_none(ignore_none, "ignore_none is required")
        validate_type(ignore_none, bool, "ignore_none must be a boolean")
        self.ignore_none = ignore_none

        self.__config_registry = None

    def write(self, data: Any) -> None:
        """
        Processes the data and writes it to the output stream.

        Args:
            data (Any): The data to be written to the output stream.

        Returns:
            None
        """
        raise NotImplementedError("OutputStream.write() not implemented")

    def close(self) -> None:
        """
        Closes the output stream.

        Args:
            None

        Returns:
            None
        """
        raise NotImplementedError("OutputStream.close() not implemented")

    def set_config_value(self, key: str, value: Any):
        """
        Sets a config value.

        Args:
            key (str): The key of the config.
            value (Any): The value to set.

        Returns:
            None
        """
        self.init_config_registry()

        if not hasattr(self.__config_registry, key):
            return

        config = next(
            (x for x in self.get_config_schema() if x.name == key),
            None)
        if config is None:
            return

        if not config.is_valid(value):
            raise ValueError(f'Invalid value for config key "{key}": {value}')

        self.__config_registry.set_value(key, value)

    def get_config_value(self, key: str) -> Any:
        """
        Gets a config value.

        Args:
            key (str): The key of the config.

        Returns:
            Any: The value of the config.
        """
        self.init_config_registry()

        if not hasattr(self.__config_registry, key):
            return None

        return self.__config_registry.get(key)

    def init_config_registry(self):
        """
        Initializes the config registry.

        Args:
            None

        Returns:
            None
        """
        if self.__config_registry is not None:
            return

        self.__config_registry = ConfigRegistry()
        for config in self.config_schema:
            self.__config_registry.set_value(config.name, config.default_value)
